from typing import List, Optional, Union
from pydantic import BaseModel, Field


class LayerInBulk(BaseModel):
    name: str = ...
    href: str = ...


class LayerDict(BaseModel):
    layer: List[LayerInBulk]


class LayersModel(BaseModel):
    layers: Union[LayerDict, str] = ""


class DefaultStyleOfLayer(BaseModel):
    name: str = ...
    href: str = ...


class ExtraStyles(BaseModel):
    class_field: str = Field(..., alias="@class")
    style: List[DefaultStyleOfLayer]


class LayerResource(BaseModel):
    class_field: str = Field(..., alias="@class")
    name: str = ...
    href: str = ...


class LayerAttribution(BaseModel):
    logoWidth: float = ...
    logoHeight: float = ...


class SingleLayer(BaseModel):
    name: str = ...
    path: str = ...
    type: str = ...
    defaultStyle: DefaultStyleOfLayer = ...
    styles: Optional[ExtraStyles] = None
    resource: LayerResource = ...
    attribution: LayerAttribution
    dateCreated: Optional[str] = None


class LayerModel(BaseModel):
    layer: SingleLayer = ...


# class NewWorkspaceInfo(BaseModel):
#     name: str = ...
#     isolated: bool = None


# class NewWorkspace(BaseModel):
#     workspace: NewWorkspaceInfo = ...



from typing import Dict, List, Optional, Union, Literal
from pydantic import BaseModel

from geoserverx.models.workspace import WorkspaceInBulk


class CoveragesStoreInBulk(BaseModel):
    name: str = ...
    href: str = ...


class CoveragesStoresDict(BaseModel):
    coverageStore: List[CoveragesStoreInBulk] = ...


class CoveragesStoresModel(BaseModel):
    coverageStores: Union[CoveragesStoresDict, Literal[""]] = ""


class CoveragesStoreModelDetail(BaseModel):
    name: str = ...
    description: str = None
    enabled: bool = ...
    workspace: WorkspaceInBulk = ...
    _default: bool = ...
    url: str = ...
    coverages: str = ...
    dateCreated: Optional[str]
    metadata: Optional[Dict] = None


class CoveragesStoreModel(BaseModel):
    coverageStore: CoveragesStoreModelDetail



from typing import List, Literal, Optional, Union
from pydantic import BaseModel, Field, ConfigDict

from .workspace import WorkspaceInBulk


class DataStoreInBulk(BaseModel):
    name: str = ...
    href: str = ...


class DataStoreDict(BaseModel):
    dataStore: List[DataStoreInBulk]


class DataStoresModel(BaseModel):
    dataStores: Union[DataStoreDict, Literal[""]] = ""


class DatastoreConnection(BaseModel):
    key: str = Field(..., alias="@key")
    path: str = Field(..., alias="$")

    model_config = ConfigDict(populate_by_name=True)


class EntryItem(BaseModel):
    entry: List[DatastoreConnection]


class DatastoreItem(BaseModel):
    name: str
    connectionParameters: EntryItem


class DataStoreModelDetails(BaseModel):
    name: str = ...
    description: str = None
    enabled: bool = ...
    workspace: WorkspaceInBulk = ...
    connectionParameters: EntryItem = ...
    _default: bool = ...
    dateCreated: Optional[str]
    dateModified: Optional[str]
    featureTypes: str


class DataStoreModel(BaseModel):
    dataStore: DataStoreModelDetails = {}


class CreateStoreItem(BaseModel):
    host: str
    port: int
    database: str
    user: str
    passwd: str
    dbtype: str


class CreateDataStoreModel(BaseModel):
    name: str
    connectionParameters: CreateStoreItem


class MainCreateDataStoreModel(BaseModel):
    dataStore: CreateDataStoreModel



from dataclasses import dataclass, field

from geoserverx.utils.errors import GeoServerXError


@dataclass
class GeoServerXAuth:
    username: str = "admin"
    password: str = "geoserver"
    url = "http://127.0.0.1:8080/geoserver/rest/"

    def __post_init__(self):
        if not self.username and not self.password and not self.url:
            raise GeoServerXError(0, "Username, Password and URL is missing")
        elif not self.username:
            raise GeoServerXError(0, "Username is missing")
        elif not self.password:
            raise GeoServerXError(0, "password is missing")
        elif not self.url:
            raise GeoServerXError(0, "URL is missing")



from typing import List, Optional
from pydantic import BaseModel


class WorkspaceInBulk(BaseModel):
    name: str = ...
    href: str = ...


class workspaceDict(BaseModel):
    workspace: List[WorkspaceInBulk]


class WorkspacesModel(BaseModel):
    workspaces: workspaceDict = ""


class SingleWorkspace(BaseModel):
    name: str = ...
    isolated: bool = ...
    dateCreated: Optional[str]
    dataStores: str = ...
    coverageStores: str = ...
    wmsStores: str = ...
    wmtsStores: str = ...


class WorkspaceModel(BaseModel):
    workspace: SingleWorkspace = ...


class NewWorkspaceInfo(BaseModel):
    name: str = ...
    isolated: bool = None


class NewWorkspace(BaseModel):
    workspace: NewWorkspaceInfo = ...



import logging
import sys


def std_out_logger(name: str) -> logging.Logger:
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter("[%(name)s] [%(levelname)s] %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger



from enum import Enum
from geoserverx.models.gs_response import GSResponse


class GSResponseEnum(Enum):
    _404 = GSResponse(code=404, response="Result not found")
    _403 = GSResponse(code=403, response="Forbidden Request")
    _401 = GSResponse(code=401, response="Unauthorized request")
    _500 = GSResponse(code=500, response="Internal Server error")
    _201 = GSResponse(code=201, response="Data added successfully")
    _200 = GSResponse(code=200, response="Executed successfully")
    _204 = GSResponse(code=204, response="No Content")
    _400 = GSResponse(code=400, response="Bad Request")
    _409 = GSResponse(code=409, response="Same data found")
    _503 = GSResponse(code=503, response="Can't connect to Geoserver")


class HTTPXErrorEnum(Enum):
    runtime = "Client not found! Please check client parameters"
    requesterr = "Client Credentials are incorrect"



from dataclasses import dataclass
from typing import Union, Optional
from geoserverx.utils.logger import std_out_logger
from geoserverx.utils.enums import GSResponseEnum
from geoserverx.models.gs_response import GSResponse
from geoserverx.utils.errors import GeoServerXError
from geoserverx.models.style import StyleModel, AllStylesModel
import asyncio
from geoserverx.utils.http_client import AsyncClient
from geoserverx.utils.auth import GeoServerXAuth
from geoserverx.models.workspace import (
    NewWorkspace,
    NewWorkspaceInfo,
    WorkspaceModel,
    WorkspacesModel,
)
from geoserverx.models.data_store import (
    DataStoreModel,
    DataStoresModel,
    CreateDataStoreModel,
    CreateStoreItem,
    MainCreateDataStoreModel,
)
from geoserverx.models.layers import LayersModel, LayerModel
from geoserverx.models.coverages_store import CoveragesStoreModel, CoveragesStoresModel
from geoserverx.models.gs_response import GSResponse
from geoserverx.utils.services.async_datastore import (
    AddDataStoreProtocol,
    CreateFileStore,
    ShapefileStore,
    GPKGfileStore,
)
import json


@dataclass
class AsyncGeoServerX:
    """
    Async Geoserver client
    """

    username: str = "admin"
    password: str = "geoserver"
    url: str = "http://127.0.0.1:8080/geoserver/rest/"
    head = {"Content-Type": "application/json"}

    def __post_init__(self):
        if not self.username and not self.password and not self.url:
            raise GeoServerXError(0, "Username, Password and URL is missing")
        elif not self.username or self.username == "":
            raise GeoServerXError(0, "Username is missing")
        elif not self.password or self.password == "":
            raise GeoServerXError(0, "password is missing")
        elif not self.url or self.url == "":
            raise GeoServerXError(0, "URL is missing")
        self.http_client = AsyncClient(
            base_url=self.url,
            auth=(self.username, self.password),
        )

    async def __aenter__(self) -> "AsyncGeoServerX":
        return self

    async def __aexit__(self, exc_t, exc_v, exc_tb) -> None:
        await self.close()

    async def close(self) -> None:
        await self.http_client.aclose()

    @staticmethod
    def from_auth(
        auth: GeoServerXAuth,
    ) -> "AsyncGeoServerX":
        return AsyncGeoServerX(auth.username, auth.password, auth.url)

    def response_recognise(self, r) -> GSResponse:
        if r == 401:
            resp = GSResponseEnum._401.value
        elif r == 500:
            resp = GSResponseEnum._500.value
        elif r == 503:
            resp = GSResponseEnum._503.value
        elif r == 404:
            resp = GSResponseEnum._404.value
        elif r == 403:
            resp = GSResponseEnum._403.value
        elif r == 201:
            resp = GSResponseEnum._201.value
        elif r == 200:
            resp = GSResponseEnum._200.value
        elif r == 409:
            resp = GSResponseEnum._409.value
        return GSResponse.model_validate(resp)

    # Get all workspaces
    async def get_all_workspaces(self) -> Union[WorkspacesModel, GSResponse]:
        Client = self.http_client
        responses = await Client.get(f"workspaces")
        if responses.status_code == 200:
            return WorkspacesModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get specific workspaces
    async def get_workspace(self, workspace: str) -> Union[WorkspaceModel, GSResponse]:
        Client = self.http_client
        responses = await Client.get(f"workspaces/{workspace}")
        if responses.status_code == 200:
            return WorkspaceModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Create workspace
    async def create_workspace(
        self, name: str, default: bool = False, Isolated: bool = False
    ) -> GSResponse:
        Client = self.http_client
        payload: NewWorkspace = NewWorkspace(
            workspace=NewWorkspaceInfo(name=name, isolated=Isolated)
        )
        responses = await Client.post(
            f"workspaces?default={default}",
            data=payload.model_dump_json(),
            headers=self.head,
        )
        results = self.response_recognise(responses.status_code)
        return results

    # Get vector stores in specific workspaces
    async def get_vector_stores_in_workspaces(self, workspace: str) -> DataStoresModel:
        Client = self.http_client
        responses = await Client.get(f"workspaces/{workspace}/datastores")
        if responses.status_code == 200:
            return DataStoresModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get raster stores in specific workspaces
    async def get_raster_stores_in_workspaces(
        self, workspace: str
    ) -> CoveragesStoresModel:
        Client = self.http_client
        responses = await Client.get(f"workspaces/{workspace}/coveragestores")
        if responses.status_code == 200:
            return CoveragesStoresModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get vector store information in specific workspaces
    async def get_vector_store(self, workspace: str, store: str) -> DataStoreModel:
        url = f"workspaces/{workspace}/datastores/{store}.json"
        Client = self.http_client
        responses = await Client.get(url)
        if responses.status_code == 200:
            return DataStoreModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get raster  store information in specific workspaces
    async def get_raster_store(self, workspace: str, store: str) -> CoveragesStoreModel:
        url = f"workspaces/{workspace}/coveragestores/{store}.json"
        Client = self.http_client
        responses = await Client.get(url)
        if responses.status_code == 200:
            return CoveragesStoreModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get all styles in GS
    async def get_allstyles(self) -> AllStylesModel:
        Client = self.http_client
        responses = await Client.get(f"styles")
        if responses.status_code == 200:
            return AllStylesModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get specific style in GS
    async def get_style(self, style: str) -> StyleModel:
        Client = self.http_client
        responses = await Client.get(f"styles/{style}.json")
        if responses.status_code == 200:
            return StyleModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Add postgres db
    async def create_pg_store(
        self,
        name: str,
        workspace: str,
        host: str,
        port: int,
        username: str,
        password: str,
        database: str,
    ) -> GSResponse:
        payload = MainCreateDataStoreModel(
            dataStore=CreateDataStoreModel(
                name=name,
                connectionParameters=CreateStoreItem(
                    host=host,
                    port=port,
                    database=database,
                    user=username,
                    passwd=password,
                    dbtype="postgis",
                ).model_dump(exclude_none=True),
            )
        )
        Client = self.http_client
        responses = await Client.post(
            f"workspaces/{workspace}/datastores/",
            data=payload.model_dump_json(),
            headers=self.head,
        )
        results = self.response_recognise(responses.status_code)
        return results

    # Get all styles in GS
    async def get_allstyles(self) -> AllStylesModel:
        Client = self.http_client
        responses = await Client.get(f"styles")
        if responses.status_code == 200:
            return AllStylesModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get specific style in GS
    async def get_style(self, style: str) -> StyleModel:
        Client = self.http_client
        responses = await Client.get(f"styles/{style}.json")
        if responses.status_code == 200:
            return StyleModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    async def create_file_store(self, workspace: str, store: str, file, service_type):
        service: AddDataStoreProtocol = CreateFileStore()

        if service_type == "shapefile":
            service = ShapefileStore(
                client=self.http_client,
                service=service,
                logger=std_out_logger("Shapefile"),
                file=file,
            )
        elif service_type == "gpkg":
            service = GPKGfileStore(
                client=self.http_client,
                service=service,
                logger=std_out_logger("GeoPackage"),
                file=file,
            )
        else:
            raise ValueError(f"Service type {service_type} not supported")
        responses = await service.addFile(self.http_client, workspace, store)
        return self.response_recognise(responses)

        if service_type == "shapefile":
            service = ShapefileStore(
                client=self.http_client,
                service=service,
                logger=std_out_logger("Shapefile"),
                file=file,
            )
        elif service_type == "gpkg":
            service = GPKGfileStore(
                service=service, logger=std_out_logger("GeoPackage"), file=file
            )
        else:
            raise ValueError(f"Service type {service_type} not supported")
        await service.addFile(self.http_client, workspace, store)

    # Get all layers
    async def get_all_layers(
        self, workspace: Optional[str] = None
    ) -> Union[LayersModel, GSResponse]:
        Client = self.http_client
        if workspace:
            responses = await Client.get(f"/workspaces/{workspace}/layers")
        else:
            responses = await Client.get(f"layers")
        if responses.status_code == 200:
            return LayersModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Get specific layer
    async def get_layer(
        self, workspace: str, layer: str
    ) -> Union[LayerModel, GSResponse]:
        Client = self.http_client
        responses = await Client.get(f"layers/{workspace}:{layer}")
        if responses.status_code == 200:
            return LayerModel.model_validate(responses.json())
        else:
            results = self.response_recognise(responses.status_code)
            return results

    # Delete specific layer
    async def delete_layer(self, workspace: str, layer: str) -> GSResponse:
        Client = self.http_client
        responses = await Client.delete(f"layers/{workspace}:{layer}")
        results = self.response_recognise(responses.status_code)
        return results



from typing import List
from pydantic import BaseModel


class langVersion(BaseModel):
    version: str = ...


class SingleStyleDict(BaseModel):
    name: str = ...
    format: str = ...
    languageVersion: langVersion = ...
    filename: str = ...


class StyleModel(BaseModel):
    style: SingleStyleDict


class allStyleList(BaseModel):
    name: str
    href: str


class allStyleDict(BaseModel):
    style: List[allStyleList]


class AllStylesModel(BaseModel):
    styles: allStyleDict



import json
from logging import Logger
from typing import Protocol
import asyncio


class AddDataStoreProtocol(Protocol):
    """
    Represents functionality of sending a file to the server.
    """

    async def addFile(
        self,
        client,
        workspace,
        store,
        method,
        store_payload,
        layer_payload,
        store_header,
        layer_header,
    ): ...


class CreateFileStore:
    async def addFile(
        self,
        client,
        workspace,
        store,
        method,
        store_payload,
        layer_payload,
        store_header,
        layer_header,
    ):
        Client = client
        # async with client as Client:
        store_responses = await Client.post(
            f"workspaces/{workspace}/datastores/",
            data=store_payload,
            headers=store_header,
        )
        # async with client as Client:
        layer_responses = await Client.put(
            f"workspaces/{workspace}/datastores/{store}/file.{method}",
            content=layer_payload,
            headers=layer_header,
        )
        results = store_responses.status_code
        # await client.aclose()

        return results


class ShapefileStore:
    def __init__(
        self, service: AddDataStoreProtocol, logger: Logger, file, client
    ) -> None:
        self.inner = service
        self.logger = logger
        self.file = file
        self.client = client

    async def addFile(self, client, workspace, store):
        store_payload: str = json.dumps(
            {
                "dataStore": {
                    "name": store,
                    "connectionParameters": {
                        "entry": [{"@key": "url", "$": "file:" + self.file}]
                    },
                }
            }
        )
        # self.logger.debug(f"Shapefile store payload: {store_payload}")
        result = await self.inner.addFile(
            self.client,
            workspace,
            store,
            "shp",
            store_payload,
            {"Content-Type": "application/json"},
            {"Content-Type": "application/zip"},
        )
        return result


class GPKGfileStore:
    def __init__(
        self, service: AddDataStoreProtocol, logger: Logger, file, client
    ) -> None:
        self.inner = service
        self.logger = logger
        self.file = file
        self.client = client

    async def addFile(self, client, workspace, store):
        store_payload: str = json.dumps(
            {
                "dataStore": {
                    "name": store,
                    "connectionParameters": {
                        "entry": [
                            {"@key": "database", "$": f"file:{self.file}"},
                            {"@key": "dbtype", "$": "geopkg"},
                        ]
                    },
                }
            }
        )
        # self.logger.debug(f"GeoPackage store payload: {store_payload}")
        result = await self.inner.addFile(
            self.client,
            workspace,
            store,
            "gpkg",
            store_payload,
            {"Content-Type": "application/json"},
        )
        return result



import typer
from enum import Enum
from geoserverx._sync.gsx import SyncGeoServerX
from rich import print
import json
from pathlib import Path

app = typer.Typer()


@app.callback()
def callback():
    """
    GeoserverX CLI tools to talk to Geoserver efficiently .
    """


# Enum for request type
class requestEnum(str, Enum):
    _sync = "sync"
    _async = "async"


# Enum for vector file type
class vectorFileEnum(str, Enum):
    shapefile = "shapefile"
    gpkg = "gpkg"


# get all workspaces
@SyncGeoServerX.exception_handler
@app.command(help="Get all workspaces in the Geoserver")
def workspaces(
    request: requestEnum = requestEnum._sync,
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get all workspaces in the Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_all_workspaces().json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# get workspace
@SyncGeoServerX.exception_handler
@app.command(help="Get workspace in the Geoserver")
def workspace(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get workspace in the Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_workspace(workspace).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# create workspace
@SyncGeoServerX.exception_handler
@app.command(help="Add workspace in the Geoserver")
def create_workspace(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    default: bool = typer.Option(False, help="Make workspace default?"),
    isolated: bool = typer.Option(False, help="Make workspace isolated?"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Add workspace in the Geoserver
    looks like - gsx create-workspace --workspace <workspacename> --default/--no-default  --isolated/--no-isolated --username <username> --password <password>
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.create_workspace(workspace, default, isolated).json()
        if json.loads(result)["code"] == 201:
            typer.secho(result, fg=typer.colors.GREEN)
        else:
            typer.secho(result, fg=typer.colors.RED)

    else:
        typer.echo("Async support will be shortly")


# Get vector stores in specific workspaces
@SyncGeoServerX.exception_handler
@app.command(help="Get vector stores in specific workspaces")
def vector_st_wp(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get vector stores in specific workspaces
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_vector_stores_in_workspaces(workspace).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# Get raster stores in specific workspaces
@SyncGeoServerX.exception_handler
@app.command(help="Get raster stores in specific workspaces")
def raster_st_wp(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get raster stores in specific workspaces
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_raster_stores_in_workspaces(workspace).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# Get vector store information in specific workspaces
@SyncGeoServerX.exception_handler
@app.command(help="Get vector store information in specific workspaces")
def vector_store(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    store: str = typer.Option(..., help="Store name"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get vector store information in specific workspaces
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_vector_store(workspace, store).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# Get raster store information in specific workspaces
@SyncGeoServerX.exception_handler
@app.command(help="Get raster store information in specific workspaces")
def raster_store(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    store: str = typer.Option(..., help="Store name"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get raster store information in specific workspaces
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_raster_store(workspace, store).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# Get all styles in Geoserver
@SyncGeoServerX.exception_handler
@app.command(help="Get all styles in Geoserver")
def styles(
    request: requestEnum = requestEnum._sync,
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get all styles in Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_allstyles().json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# Get style in Geoserver
@SyncGeoServerX.exception_handler
@app.command(help="Get style in Geoserver")
def style(
    request: requestEnum = requestEnum._sync,
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    style: str = typer.Option(..., help="Style name"),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get style in Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_style(style).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# Create Vector Layer in Geoserver
@SyncGeoServerX.exception_handler
@app.command(help="Create Vector Layer in Geoserver")
def create_file(
    request: requestEnum = requestEnum._sync,
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    workspace: str = typer.Option(..., help="Workspace name"),
    store: str = typer.Option(..., help="Store name"),
    service_type: vectorFileEnum = typer.Option(..., help="Vector file type"),
    file: Path = typer.Option(..., help="File path"),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Create Vector Layer in Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        try:
            files = open(file, "rb")
            result = client.create_file_store(
                workspace, store, files.read(), service_type
            )
            if result.code == 201:
                typer.secho(result, fg=typer.colors.GREEN)
            else:
                typer.secho(result, fg=typer.colors.RED)
        except:
            typer.secho("File path is incorrect", fg=typer.colors.YELLOW)
    else:
        typer.echo("Async support will be shortly")


# Create PostgreSQL store in Geoserver
@SyncGeoServerX.exception_handler
@app.command(help="Create PostgreSQL store in Geoserver")
def create_pg_store(
    request: requestEnum = requestEnum._sync,
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    name: str = typer.Option(..., help="Store name"),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
    workspace: str = typer.Option(..., help="workspace name"),
    host: str = typer.Option("localhost", help="Host IP Address"),
    port: int = typer.Option(5432, help="Database port"),
    dbuser: str = typer.Option("postgres", help="Database username"),
    dbname: str = typer.Option(..., help="Database name"),
    dbpwd: str = typer.Option(..., help="Database password"),
):
    """
    Create PostgreSQL store in Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.create_pg_store(
            name=name,
            workspace=workspace,
            host=host,
            port=port,
            username=dbuser,
            password=dbpwd,
            database=dbname,
        )
        if result.code == 201:
            typer.secho(result, fg=typer.colors.GREEN)
        else:
            typer.secho(result, fg=typer.colors.RED)
    else:
        typer.echo("Async support will be shortly")


# get all layers
@SyncGeoServerX.exception_handler
@app.command(help="Get all layers in the Geoserver")
def layers(
    request: requestEnum = requestEnum._sync,
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    workspace: str = typer.Option(None, help="Workspace name"),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get all layers in the Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_all_layers(workspace).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")


# get layer
@SyncGeoServerX.exception_handler
@app.command(help="Get layer in the Geoserver")
def layer(
    request: requestEnum = requestEnum._sync,
    workspace: str = typer.Option(..., help="Workspace name"),
    layer: str = typer.Option(..., help="Layer name"),
    url: str = typer.Option(
        "http://127.0.0.1:8080/geoserver/rest/", help="Geoserver REST URL"
    ),
    password: str = typer.Option("geoserver", help="Geoserver Password"),
    username: str = typer.Option("admin", help="Geoserver username"),
):
    """
    Get workspace in the Geoserver
    """
    if request.value == "sync":
        client = SyncGeoServerX(username, password, url)
        result = client.get_layer(workspace, layer).json()
        if "code" in result:
            typer.secho(result, fg=typer.colors.RED)
        else:
            print(result)
    else:
        typer.echo("Async support will be shortly")
